@@ -1,22 +1,31 @@
 #!/usr/bin/env python
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 from __future__ import print_function
-from . import _version
 
-import sys
-import requests
-import json
 import base64
+import io
+import json
+import logging
+import os
+import re
+import sys
+
+import requests
+from PIL import Image
+from distutils.spawn import find_executable
 from bs4 import BeautifulSoup
-from six.moves import urllib_parse, input
+from requests import HTTPError
 from six import print_ as print
+from six.moves import urllib_parse, input
+
+from aws_google_auth import _version
 
 # The U2F USB Library is optional, if it's there, include it.
 try:
-    from . import u2f
+    from aws_google_auth import u2f
 except ImportError:
-    print("Failed to import U2F libraries, U2F login unavailable. Other "
-          "methods can still continue.")
+    logging.info("Failed to import U2F libraries, U2F login unavailable. "
+                 "Other methods can still continue.")
 
 
 class ExpectedGoogleException(Exception):
@@ -25,7 +34,7 @@ class ExpectedGoogleException(Exception):
 
 
 class Google:
-    def __init__(self, config):
+    def __init__(self, config, save_failure):
         """The Google object holds authentication state
         for a given session. You need to supply:
 
@@ -41,14 +50,15 @@ class Google:
         self.version = _version.__version__
         self.config = config
         self.base_url = 'https://accounts.google.com'
+        self.save_failure = save_failure
+        self.session_state = None
 
     @property
     def login_url(self):
         return self.base_url + "/o/saml2/initsso?idpid={}&spid={}&forceauthn=false".format(
             self.config.idp_id, self.config.sp_id)
 
-    @staticmethod
-    def check_for_failure(sess):
+    def check_for_failure(self, sess):
 
         if isinstance(sess.reason, bytes):
             # We attempt to decode utf-8 first because some servers
@@ -66,25 +76,32 @@ class Google:
             raise ExpectedGoogleException(u'{} accessing {}'.format(
                 reason, sess.url))
 
-        sess.raise_for_status()
+        try:
+            sess.raise_for_status()
+        except HTTPError as ex:
+
+            if self.save_failure:
+                logging.exception("Saving failure trace in 'failure.html'", ex)
+                with open("failure.html", 'w') as out:
+                    out.write(sess.text)
+
+            raise ex
 
         return sess
 
     def post(self, url, data=None, json=None):
         try:
-            response = self.check_for_failure(
-                self.session.post(url, data=data, json=json))
+            response = self.check_for_failure(self.session.post(url, data=data, json=json))
         except requests.exceptions.ConnectionError as e:
-            print(
-                'There was a connection error, check your network settings: {}'.
-                format(e))
+            logging.exception(
+                'There was a connection error, check your network settings.', e)
             sys.exit(1)
         except requests.exceptions.Timeout as e:
-            print('The connection timed out, please try again: {}'.format(e))
+            logging.exception('The connection timed out, please try again.', e)
             sys.exit(1)
         except requests.exceptions.TooManyRedirects as e:
-            print('The number of redirects exceeded the maximum allowed: {}'.
-                  format(e))
+            logging.exception('The number of redirects exceeded the maximum '
+                              'allowed.', e)
             sys.exit(1)
 
         return response
@@ -93,16 +110,15 @@ class Google:
         try:
             response = self.check_for_failure(self.session.get(url))
         except requests.exceptions.ConnectionError as e:
-            print(
-                'There was a connection error, check your network settings: {}'.
-                format(e))
+            logging.exception(
+                'There was a connection error, check your network settings.', e)
             sys.exit(1)
         except requests.exceptions.Timeout as e:
-            print('The connection timed out, please try again: {}'.format(e))
+            logging.exception('The connection timed out, please try again.', e)
             sys.exit(1)
         except requests.exceptions.TooManyRedirects as e:
-            print('The number of redirects exceeded the maximum allowed: {}'.
-                  format(e))
+            logging.exception('The number of redirects exceeded the maximum '
+                              'allowed.', e)
             sys.exit(1)
 
         return response
@@ -117,11 +133,45 @@ class Google:
         else:
             return error.text
 
+    @staticmethod
+    def find_key_handles(input, challengeTxt):
+        keyHandles = []
+        typeOfInput = type(input)
+        if typeOfInput == dict:  # parse down a dict
+            for item in input:
+                keyHandles.extend(Google.find_key_handles(input[item], challengeTxt))
+
+        elif typeOfInput == list:  # looks like we've hit an array - iterate it
+            array = list(filter(None, input))  # remove any None type objects from the array
+            for item in array:
+                typeValue = type(item)
+                if typeValue == list:  # another array - recursive call
+                    keyHandles.extend(Google.find_key_handles(item, challengeTxt))
+                elif typeValue == int or typeValue == bool:  # ints bools etc we don't care
+                    continue
+                else:  # we went a string or unicode here (python 3.x lost unicode global)
+                    try:  # keyHandle string will be base64 encoded -
+                        # if its not an exception is thrown and we continue as its not the string we're after
+                        base64UrlEncoded = base64.urlsafe_b64encode(base64.b64decode(item))
+                        if base64UrlEncoded != challengeTxt:  # make sure its not the challengeTxt - if it not return it
+                            keyHandles.append(base64UrlEncoded)
+                    except:
+                        pass
+        return keyHandles
+
+    @staticmethod
+    def find_app_id(inputString):
+        try:
+            searchResult = re.search('"appid":"[a-z://.-_]+"', inputString).group()
+            searchObject = json.loads('{' + searchResult + '}')
+            return str(searchObject['appid'])
+        except:
+            logging.exception('Was unable to find appid value in googles SAML page')
+            sys.exit(1)
+
     def do_login(self):
         self.session = requests.Session()
-        self.session.headers[
-            'User-Agent'] = "AWS Sign-in/{} (Cevo aws-google-auth)".format(
-                self.version)
+        self.session.headers['User-Agent'] = "AWS Sign-in/{} (aws-google-auth)".format(self.version)
         sess = self.get(self.login_url)
 
         # Collect information from the page source
@@ -130,9 +180,7 @@ class Google:
         self.cont = first_page.find('input', {'name': 'continue'}).get('value')
         page = first_page.find('input', {'name': 'Page'}).get('value')
         sign_in = first_page.find('input', {'name': 'signIn'}).get('value')
-        account_login_url = first_page.find('form', {
-            'id': 'gaia_loginform'
-        }).get('action')
+        account_login_url = first_page.find('form', {'id': 'gaia_loginform'}).get('action')
 
         payload = {
             'bgresponse': 'js_disabled',
@@ -158,6 +206,9 @@ class Google:
             '_utf8': '?',
         }
 
+        if self.config.bg_response:
+            payload['bgresponse'] = self.config.bg_response
+
         # GALX is sometimes not there
         try:
             galx = first_page.find('input', {'name': 'GALX'}).get('value')
@@ -173,21 +224,23 @@ class Google:
         # Collect ProfileInformation, SessionState, signIn, and Password Challenge URL
         challenge_page = BeautifulSoup(sess.text, 'html.parser')
 
-        profile_information = challenge_page.find('input', {
-            'name': 'ProfileInformation'
-        }).get('value')
-        session_state = challenge_page.find('input', {
-            'name': 'SessionState'
-        }).get('value')
-        sign_in = challenge_page.find('input', {'name': 'signIn'}).get('value')
-        passwd_challenge_url = challenge_page.find('form', {
-            'id': 'gaia_loginform'
-        }).get('action')
+        # Handle the "old-style" page
+        if challenge_page.find('form', {'id': 'gaia_loginform'}):
+            form = challenge_page.find('form', {'id': 'gaia_loginform'})
+            passwd_challenge_url = form.get('action')
+        else:
+            # sometimes they serve up a different page
+            logging.info("Handling new-style login page")
+            form = challenge_page.find('form', {'id': 'challenge'})
+            passwd_challenge_url = 'https://accounts.google.com' + form.get('action')
+
+        for tag in form.find_all('input'):
+            if tag.get('name') is None:
+                continue
+
+            payload[tag.get('name')] = tag.get('value')
 
         # Update the payload
-        payload['SessionState'] = session_state
-        payload['ProfileInformation'] = profile_information
-        payload['signIn'] = sign_in
         payload['Passwd'] = self.config.password
 
         # POST to Authenticate Password
@@ -195,19 +248,42 @@ class Google:
 
         response_page = BeautifulSoup(sess.text, 'html.parser')
         error = response_page.find(class_='error-msg')
-        cap = response_page.find('input', {'name': 'logincaptcha'})
+        cap = response_page.find('input', {'name': 'identifier-captcha-input'})
 
         # Were there any errors logging in? Could be invalid username or password
         # There could also sometimes be a Captcha, which means Google thinks you,
         # or someone using the same outbound IP address as you, is a bot.
-        if error is not None:
+        if error is not None and cap is None:
             raise ExpectedGoogleException('Invalid username or password')
 
+        if "signin/rejected" in sess.url:
+            raise ExpectedGoogleException(u'''Default value of parameter `bgresponse` has not accepted.
+                Please visit login URL {}, open the web inspector and execute document.bg.invoke() in the console.
+                Then, set --bg-response to the function output.'''.format(self.login_url))
+
         self.check_extra_step(response_page)
 
+        # Process Google CAPTCHA verification request if present
         if cap is not None:
-            raise ExpectedGoogleException(
-                'Captcha Required. Manually Login to remove this.')
+            self.session.headers['Referer'] = sess.url
+
+            sess = self.handle_captcha(sess, payload)
+
+            response_page = BeautifulSoup(sess.text, 'html.parser')
+            error = response_page.find(class_='error-msg')
+            cap = response_page.find('input', {'name': 'logincaptcha'})
+
+            # Were there any errors logging in? Could be invalid username or password
+            # There could also sometimes be a Captcha, which means Google thinks you,
+            # or someone using the same outbound IP address as you, is a bot.
+            if error is not None:
+                raise ExpectedGoogleException('Invalid username or password')
+
+            self.check_extra_step(response_page)
+
+            if cap is not None:
+                raise ExpectedGoogleException(
+                    'Invalid captcha')
 
         self.session.headers['Referer'] = sess.url
 
@@ -221,7 +297,7 @@ class Google:
                 sess = self.handle_totp(sess)
                 error_msg = self.parse_error_message(sess)
                 if error_msg is not None:
-                    print(error_msg)
+                    logging.error(error_msg)
         elif "challenge/ipp/" in sess.url:
             sess = self.handle_sms(sess)
         elif "challenge/az/" in sess.url:
@@ -241,56 +317,161 @@ class Google:
 
     @staticmethod
     def check_extra_step(response):
-        extra_step = response.find(
-            text='This extra step shows that it’s really you trying to sign in'
-        )
+        extra_step = response.find(text='This extra step shows that it’s really you trying to sign in')
         if extra_step:
             if response.find(id='contactAdminMessage'):
                 raise ValueError(response.find(id='contactAdminMessage').text)
 
     def parse_saml(self):
         if self.session_state is None:
-            raise RuntimeError(
-                'You must use do_login() before calling parse_saml()')
+            raise RuntimeError('You must use do_login() before calling parse_saml()')
 
         parsed = BeautifulSoup(self.session_state.text, 'html.parser')
         try:
-            saml_element = parsed.find('input', {
-                'name': 'SAMLResponse'
-            }).get('value')
+            saml_element = parsed.find('input', {'name': 'SAMLResponse'}).get('value')
         except:
-            raise RuntimeError(
-                'Could not find SAML response, check your credentials')
+
+            if self.save_failure:
+                logging.error("SAML lookup failed, storing failure page to "
+                              "'saml.html' to assist with debugging.")
+                with open("saml.html", 'w') as out:
+                    out.write(str(self.session_state.text.encode('utf-8')))
+
+            raise ExpectedGoogleException('Something went wrong - Could not find SAML response, check your credentials or use --save-failure-html to debug.')
 
         return base64.b64decode(saml_element)
 
+    def handle_captcha(self, sess, payload):
+        response_page = BeautifulSoup(sess.text, 'html.parser')
+
+        # Collect ProfileInformation, SessionState, signIn, and Password Challenge URL
+        profile_information = response_page.find('input', {
+            'name': 'ProfileInformation'
+        }).get('value')
+        session_state = response_page.find('input', {
+            'name': 'SessionState'
+        }).get('value')
+        sign_in = response_page.find('input', {'name': 'signIn'}).get('value')
+        passwd_challenge_url = response_page.find('form', {
+            'id': 'gaia_loginform'
+        }).get('action')
+
+        # Update the payload
+        payload['SessionState'] = session_state
+        payload['ProfileInformation'] = profile_information
+        payload['signIn'] = sign_in
+        payload['Passwd'] = self.config.password
+
+        # Get all captcha challenge tokens and urls
+        captcha_container = response_page.find('div', {'id': 'identifier-captcha'})
+        captcha_logintoken = captcha_container.find('input', {'id': 'identifier-token'}).get('value')
+        captcha_img = captcha_container.find('div', {'class': 'captcha-img'})
+        captcha_url = "https://accounts.google.com" + captcha_img.find('img').get('src')
+        captcha_logintoken_audio = ''
+
+        open_image = True
+
+        # Check if there is a display utility installed as Image.open(f).show() do not raise any exception if not
+        # if neither xv or display are available just display the URL for the user to visit.
+        if os.name == 'posix' and sys.platform != 'darwin':
+            if find_executable('xv') is None and find_executable('display') is None:
+                open_image = False
+
+        print("Please visit the following URL to view your CAPTCHA: {}".format(captcha_url))
+
+        if open_image:
+            try:
+                with requests.get(captcha_url) as url:
+                    with io.BytesIO(url.content) as f:
+                        Image.open(f).show()
+            except Exception:
+                pass
+
+        try:
+            captcha_input = raw_input("Captcha (case insensitive): ") or None
+        except NameError:
+            captcha_input = input("Captcha (case insensitive): ") or None
+
+        # Update the payload
+        payload['identifier-captcha-input'] = captcha_input
+        payload['identifiertoken'] = captcha_logintoken
+        payload['identifiertoken_audio'] = captcha_logintoken_audio
+        payload['checkedDomains'] = 'youtube'
+        payload['checkConnection'] = 'youtube:574:1'
+        payload['Email'] = self.config.username
+
+        response = self.post(passwd_challenge_url, data=payload)
+
+        newPayload = {}
+
+        auth_response_page = BeautifulSoup(response.text, 'html.parser')
+
+        challengeId = auth_response_page.find('input', {
+            'name': 'challengeId'
+        }).get('value')
+        challengeType = auth_response_page.find('input', {
+            'name': 'challengeType'
+        }).get('value')
+        tl = auth_response_page.find('input', {
+            'name': 'TL'
+        }).get('value')
+        gxf = auth_response_page.find('input', {
+            'name': 'gxf'
+        }).get('value')
+
+        newPayload['challengeId'] = challengeId
+        newPayload['challengeType'] = challengeType
+        newPayload['TL'] = tl
+        newPayload['gxf'] = gxf
+        newPayload['TrustDevice'] = "on"
+        newPayload['continue'] = payload['continue']
+        newPayload['ltmpl'] = payload['ltmpl']
+        newPayload['scc'] = payload['scc']
+        newPayload['sarp'] = payload['sarp']
+        newPayload['checkedDomains'] = payload['checkedDomains']
+        newPayload['checkConnection'] = payload['checkConnection']
+        newPayload['pstMsg'] = payload['pstMsg']
+        newPayload['oauth'] = payload['oauth']
+        newPayload['Email'] = payload['Email']
+        newPayload['Passwd'] = payload['Passwd']
+
+        return self.post(response.url, data=newPayload)
+
     def handle_sk(self, sess):
         response_page = BeautifulSoup(sess.text, 'html.parser')
         challenge_url = sess.url.split("?")[0]
-
         challenges_txt = response_page.find('input', {
             'name': "id-challenge"
         }).get('value')
-        challenges = json.loads(challenges_txt)
 
         facet_url = urllib_parse.urlparse(challenge_url)
         facet = facet_url.scheme + "://" + facet_url.netloc
-        app_id = challenges["appId"]
-        u2f_challenges = []
-        for c in challenges["challenges"]:
-            c["appId"] = app_id
-            u2f_challenges.append(c)
+
+        keyHandleJSField = response_page.find('div', {'jsname': 'C0oDBd'}).get('data-challenge-ui')
+        startJSONPosition = keyHandleJSField.find('{')
+        endJSONPosition = keyHandleJSField.rfind('}')
+        keyHandleJsonPayload = json.loads(keyHandleJSField[startJSONPosition:endJSONPosition + 1])
+
+        keyHandles = self.find_key_handles(keyHandleJsonPayload, base64.urlsafe_b64encode(base64.b64decode(challenges_txt)))
+        appId = self.find_app_id(str(keyHandleJsonPayload))
+
+        # txt sent for signing needs to be base64 url encode
+        # we also have to remove any base64 padding because including including it will prevent google accepting the auth response
+        challenges_txt_encode_pad_removed = base64.urlsafe_b64encode(base64.b64decode(challenges_txt)).strip('='.encode())
+
+        u2f_challenges = [{'version': 'U2F_V2', 'challenge': challenges_txt_encode_pad_removed.decode(), 'appId': appId, 'keyHandle': keyHandle.decode()} for keyHandle in keyHandles]
 
         # Prompt the user up to attempts_remaining times to insert their U2F device.
         attempts_remaining = 5
         auth_response = None
         while True:
             try:
-                auth_response = json.dumps(u2f.u2f_auth(u2f_challenges, facet))
+                auth_response_dict = u2f.u2f_auth(u2f_challenges, facet)
+                auth_response = json.dumps(auth_response_dict)
                 break
             except RuntimeWarning:
-                print("No U2F device found. {} attempts remaining.".format(
-                    attempts_remaining))
+                logging.error("No U2F device found. %d attempts remaining",
+                              attempts_remaining)
                 if attempts_remaining <= 0:
                     break
                 else:
@@ -313,8 +494,7 @@ class Google:
             response_page.find('input', {
                 'name': 'challengeType'
             }).get('value'),
-            'continue':
-            response_page.find('input', {
+            'continue': response_page.find('input', {
                 'name': 'continue'
             }).get('value'),
             'scc':
@@ -329,10 +509,7 @@ class Google:
             response_page.find('input', {
                 'name': 'checkedDomains'
             }).get('value'),
-            'pstMsg':
-            response_page.find('input', {
-                'name': 'pstMsg'
-            }).get('value'),
+            'pstMsg': '1',
             'TL':
             response_page.find('input', {
                 'name': 'TL'
@@ -420,13 +597,22 @@ class Google:
 
         self.check_prompt_code(response_page)
 
-        print(
-            "Open the Google App, and tap 'Yes' on the prompt to sign in ...")
+        print("Open the Google App, and tap 'Yes' on the prompt to sign in ...")
 
         self.session.headers['Referer'] = sess.url
 
-        parsed_response = json.loads(
-            self.post(await_url, json=await_body).text)
+        retry = True
+        response = None
+        while retry:
+            try:
+                response = self.post(await_url, json=await_body)
+                retry = False
+            except requests.exceptions.HTTPError as ex:
+
+                if not ex.response.status_code == 500:
+                    raise ex
+
+        parsed_response = json.loads(response.text)
 
         payload = {
             'challengeId':
@@ -534,7 +720,7 @@ class Google:
                 if choice not in [1, 2]:
                     raise ValueError
             except ValueError:
-                print("Not a valid (integer) option, try again")
+                logging.error("Not a valid (integer) option, try again")
                 continue
             else:
                 if choice == 1:
@@ -679,8 +865,7 @@ class Google:
         selected_challenge = input("Enter MFA choice number ({}): ".format(
             challenge_ids[-1:][0])) or None
 
-        if selected_challenge is not None and int(
-                selected_challenge) in challenge_ids:
+        if selected_challenge is not None and int(selected_challenge) in challenge_ids:
             challenge_id = int(selected_challenge)
         else:
             # use the highest index as that will default to prompt, then sms, then totp, etc.
