@@ -1,9 +1,9 @@
-import aws_google_auth
-
 import unittest
+from argparse import Namespace
 
 from mock import call, patch, Mock, MagicMock
-from argparse import Namespace
+
+import aws_google_auth
 
 
 class TestInit(unittest.TestCase):
@@ -48,14 +48,22 @@ class TestInit(unittest.TestCase):
                                          keyring=False,
                                          disable_u2f=False,
                                          duration=None,
+                                         auto_duration=False,
                                          idp_id=None,
                                          profile=None,
                                          region=None,
                                          resolve_aliases=False,
                                          role_arn=None,
+                                         save_failure_html=False,
                                          saml_cache=True,
+                                         saml_assertion=None,
                                          sp_id=None,
-                                         username=None))
+                                         log_level='warn',
+                                         print_creds=False,
+                                         username=None,
+                                         quiet=False,
+                                         bg_response=None,
+                                         account=None))
                           ],
                          resolve_config.mock_calls)
 
@@ -63,23 +71,30 @@ class TestInit(unittest.TestCase):
                                          keyring=False,
                                          disable_u2f=False,
                                          duration=None,
+                                         auto_duration=False,
                                          idp_id=None,
                                          profile=None,
                                          region=None,
                                          resolve_aliases=False,
                                          role_arn=None,
+                                         save_failure_html=False,
                                          saml_cache=True,
+                                         saml_assertion=None,
                                          sp_id=None,
-                                         username=None),
+                                         log_level='warn',
+                                         print_creds=False,
+                                         username=None,
+                                         quiet=False,
+                                         bg_response=None,
+                                         account=None),
                                mock_config)
                           ],
                          process_auth.mock_calls)
 
-    @patch('getpass.getpass', spec=True)
     @patch('aws_google_auth.util', spec=True)
     @patch('aws_google_auth.amazon', spec=True)
     @patch('aws_google_auth.google', spec=True)
-    def test_process_auth_standard(self, mock_google, mock_amazon, mock_util, mock_getpass):
+    def test_process_auth_standard(self, mock_google, mock_amazon, mock_util):
 
         mock_config = Mock()
         mock_config.profile = False
@@ -89,11 +104,85 @@ class TestInit(unittest.TestCase):
         mock_config.idp_id = None
         mock_config.sp_id = None
         mock_config.return_value = None
+        mock_config.account = None
+        mock_config.region = None
 
         mock_amazon_client = Mock()
         mock_google_client = Mock()
 
-        mock_getpass.return_value = "pass"
+        mock_amazon_client.roles = {
+            'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+            'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
+        }
+
+        mock_util_obj = MagicMock()
+        mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
+        mock_util_obj.get_input = MagicMock(side_effect=["region_input", "input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
+
+        mock_util.Util = mock_util_obj
+
+        mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
+
+        mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
+        mock_google.Google = MagicMock(return_value=mock_google_client)
+
+        args = aws_google_auth.parse_args([])
+
+        # Method Under Test
+        aws_google_auth.process_auth(args, mock_config)
+
+        # Assert values collected
+        self.assertEqual(mock_config.region, "region_input")
+        self.assertEqual(mock_config.username, "input")
+        self.assertEqual(mock_config.idp_id, "input2")
+        self.assertEqual(mock_config.sp_id, "input3")
+        self.assertEqual(mock_config.password, "pass")
+        self.assertEqual(mock_config.provider, "da_provider")
+        self.assertEqual(mock_config.role_arn, "da_role")
+
+        # Assert calls occur
+        self.assertEqual([call.Util.get_input('AWS Region: '),
+                          call.Util.get_input('Google username: '),
+                          call.Util.get_input('Google IDP ID: '),
+                          call.Util.get_input('Google SP ID: '),
+                          call.Util.get_password('Google Password: '),
+                          call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+                                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
+                         mock_util.mock_calls)
+
+        self.assertEqual([call.do_login(), call.parse_saml()],
+                         mock_google_client.mock_calls)
+
+        self.assertEqual([call.raise_if_invalid()],
+                         mock_config.mock_calls)
+
+        self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
+                                })],
+                         mock_amazon_client.resolve_aws_aliases.mock_calls)
+
+        self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
+                          ], mock_util_obj.pick_a_role.mock_calls)
+
+    @patch('aws_google_auth.util', spec=True)
+    @patch('aws_google_auth.amazon', spec=True)
+    @patch('aws_google_auth.google', spec=True)
+    def test_process_auth_print_creds(self, mock_google, mock_amazon, mock_util):
+        mock_config = Mock()
+        mock_config.profile = False
+        mock_config.saml_cache = False
+        mock_config.keyring = False
+        mock_config.username = None
+        mock_config.idp_id = None
+        mock_config.sp_id = None
+        mock_config.return_value = None
+        mock_config.print_creds = True
+        mock_config.account = None
+
+        mock_amazon_client = Mock()
+        mock_google_client = Mock()
 
         mock_amazon_client.roles = {
             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
@@ -103,6 +192,7 @@ class TestInit(unittest.TestCase):
         mock_util_obj = MagicMock()
         mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
         mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
 
         mock_util.Util = mock_util_obj
 
@@ -129,36 +219,36 @@ class TestInit(unittest.TestCase):
         self.assertEqual([call.Util.get_input('Google username: '),
                           call.Util.get_input('Google IDP ID: '),
                           call.Util.get_input('Google SP ID: '),
+                          call.Util.get_password('Google Password: '),
                           call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
-                                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
+                                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'},
+                                                [])],
                          mock_util.mock_calls)
 
-        self.assertEqual([call()],
-                         mock_amazon_client.print_export_line.mock_calls)
-
-        self.assertEqual([call('Google Password: ')],
-                         mock_getpass.mock_calls)
-
         self.assertEqual([call.do_login(), call.parse_saml()],
                          mock_google_client.mock_calls)
 
         self.assertEqual([call.raise_if_invalid()],
                          mock_config.mock_calls)
 
-        self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
-                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
-                                })],
-                         mock_amazon_client.resolve_aws_aliases.mock_calls)
+        self.assertEqual(
+            [call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+                   'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
+                   })],
+            mock_amazon_client.resolve_aws_aliases.mock_calls)
 
-        self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
-                                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
-                          ], mock_util_obj.pick_a_role.mock_calls)
+        self.assertEqual(
+            [call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
+                   'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
+             ], mock_util_obj.pick_a_role.mock_calls)
+
+        self.assertEqual([call()],
+                         mock_amazon_client.print_export_line.mock_calls)
 
-    @patch('getpass.getpass', spec=True)
     @patch('aws_google_auth.util', spec=True)
     @patch('aws_google_auth.amazon', spec=True)
     @patch('aws_google_auth.google', spec=True)
-    def test_process_auth_specified_role(self, mock_google, mock_amazon, mock_util, mock_getpass):
+    def test_process_auth_specified_role(self, mock_google, mock_amazon, mock_util):
 
         mock_config = Mock()
         mock_config.saml_cache = False
@@ -174,8 +264,6 @@ class TestInit(unittest.TestCase):
         mock_amazon_client = Mock()
         mock_google_client = Mock()
 
-        mock_getpass.return_value = "pass"
-
         mock_amazon_client.roles = {
             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
             'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
@@ -184,6 +272,7 @@ class TestInit(unittest.TestCase):
         mock_util_obj = MagicMock()
         mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
         mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
 
         mock_util.Util = mock_util_obj
 
@@ -208,12 +297,10 @@ class TestInit(unittest.TestCase):
         # Assert calls occur
         self.assertEqual([call.Util.get_input('Google username: '),
                           call.Util.get_input('Google IDP ID: '),
-                          call.Util.get_input('Google SP ID: ')],
+                          call.Util.get_input('Google SP ID: '),
+                          call.Util.get_password('Google Password: ')],
                          mock_util.mock_calls)
 
-        self.assertEqual([call('Google Password: ')],
-                         mock_getpass.mock_calls)
-
         self.assertEqual([call.do_login(), call.parse_saml()],
                          mock_google_client.mock_calls)
 
@@ -227,11 +314,10 @@ class TestInit(unittest.TestCase):
         self.assertEqual([],
                          mock_util_obj.pick_a_role.mock_calls)
 
-    @patch('getpass.getpass', spec=True)
     @patch('aws_google_auth.util', spec=True)
     @patch('aws_google_auth.amazon', spec=True)
     @patch('aws_google_auth.google', spec=True)
-    def test_process_auth_dont_resolve_alias(self, mock_google, mock_amazon, mock_util, mock_getpass):
+    def test_process_auth_dont_resolve_alias(self, mock_google, mock_amazon, mock_util):
 
         mock_config = Mock()
         mock_config.saml_cache = False
@@ -241,12 +327,11 @@ class TestInit(unittest.TestCase):
         mock_config.sp_id = None
         mock_config.return_value = None
         mock_config.keyring = False
+        mock_config.account = None
 
         mock_amazon_client = Mock()
         mock_google_client = Mock()
 
-        mock_getpass.return_value = "pass"
-
         mock_amazon_client.roles = {
             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
             'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
@@ -255,6 +340,7 @@ class TestInit(unittest.TestCase):
         mock_util_obj = MagicMock()
         mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
         mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
 
         mock_util.Util = mock_util_obj
 
@@ -275,18 +361,17 @@ class TestInit(unittest.TestCase):
         self.assertEqual(mock_config.password, "pass")
         self.assertEqual(mock_config.provider, "da_provider")
         self.assertEqual(mock_config.role_arn, "da_role")
+        self.assertEqual(mock_config.account, None)
 
         # Assert calls occur
         self.assertEqual([call.Util.get_input('Google username: '),
                           call.Util.get_input('Google IDP ID: '),
                           call.Util.get_input('Google SP ID: '),
+                          call.Util.get_password('Google Password: '),
                           call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
                                                 'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'})],
                          mock_util.mock_calls)
 
-        self.assertEqual([call('Google Password: ')],
-                         mock_getpass.mock_calls)
-
         self.assertEqual([call.do_login(), call.parse_saml()],
                          mock_google_client.mock_calls)
 
@@ -301,11 +386,10 @@ class TestInit(unittest.TestCase):
                                 'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'})
                           ], mock_util_obj.pick_a_role.mock_calls)
 
-    @patch('getpass.getpass', spec=True)
     @patch('aws_google_auth.util', spec=True)
     @patch('aws_google_auth.amazon', spec=True)
     @patch('aws_google_auth.google', spec=True)
-    def test_process_auth_with_profile(self, mock_google, mock_amazon, mock_util, mock_getpass):
+    def test_process_auth_with_profile(self, mock_google, mock_amazon, mock_util):
 
         mock_config = Mock()
         mock_config.saml_cache = False
@@ -316,12 +400,11 @@ class TestInit(unittest.TestCase):
         mock_config.profile = "blart"
         mock_config.return_value = None
         mock_config.role_arn = 'arn:aws:iam::123456789012:role/admin'
+        mock_config.account = None
 
         mock_amazon_client = Mock()
         mock_google_client = Mock()
 
-        mock_getpass.return_value = "pass"
-
         mock_amazon_client.roles = {
             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
             'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
@@ -330,6 +413,7 @@ class TestInit(unittest.TestCase):
         mock_util_obj = MagicMock()
         mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
         mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
 
         mock_util.Util = mock_util_obj
 
@@ -355,13 +439,11 @@ class TestInit(unittest.TestCase):
         self.assertEqual([call.Util.get_input('Google username: '),
                           call.Util.get_input('Google IDP ID: '),
                           call.Util.get_input('Google SP ID: '),
+                          call.Util.get_password('Google Password: '),
                           call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
                                                 'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
                          mock_util.mock_calls)
 
-        self.assertEqual([call('Google Password: ')],
-                         mock_getpass.mock_calls)
-
         self.assertEqual([call.do_login(), call.parse_saml()],
                          mock_google_client.mock_calls)
 
@@ -378,11 +460,10 @@ class TestInit(unittest.TestCase):
                                 'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
                           ], mock_util_obj.pick_a_role.mock_calls)
 
-    @patch('getpass.getpass', spec=True)
     @patch('aws_google_auth.util', spec=True)
     @patch('aws_google_auth.amazon', spec=True)
     @patch('aws_google_auth.google', spec=True)
-    def test_process_auth_with_saml_cache(self, mock_google, mock_amazon, mock_util, mock_getpass):
+    def test_process_auth_with_saml_cache(self, mock_google, mock_amazon, mock_util):
 
         mock_config = Mock()
         mock_config.saml_cache = True
@@ -392,12 +473,11 @@ class TestInit(unittest.TestCase):
         mock_config.password = None
         mock_config.return_value = None
         mock_config.role_arn = 'arn:aws:iam::123456789012:role/admin'
+        mock_config.account = None
 
         mock_amazon_client = Mock()
         mock_google_client = Mock()
 
-        mock_getpass.return_value = "pass"
-
         mock_amazon_client.roles = {
             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
             'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
@@ -406,6 +486,7 @@ class TestInit(unittest.TestCase):
         mock_util_obj = MagicMock()
         mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
         mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
+        mock_util_obj.get_password = MagicMock(return_value="pass")
 
         mock_util.Util = mock_util_obj
 
@@ -432,10 +513,6 @@ class TestInit(unittest.TestCase):
                                                 'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
                          mock_util.mock_calls)
 
-        # Cache means no password request
-        self.assertEqual([],
-                         mock_getpass.mock_calls)
-
         # Cache means no google calls
         self.assertEqual([],
                          mock_google_client.mock_calls)
